Source-Changes-HG archive

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index][Old Index]

[src/trunk]: src/sys/dev Disestablish COW handler on error. No need to do fu...



details:   https://anonhg.NetBSD.org/src/rev/267012406009
branches:  trunk
changeset: 346739:267012406009
user:      hannken <hannken%NetBSD.org@localhost>
date:      Sun Jul 31 12:17:36 2016 +0000

description:
Disestablish COW handler on error.  No need to do further copies after
the snapshot device failed.

Should fix PR kern/51377: fss(4) panic if snapshot mounted read/write

diffstat:

 sys/dev/fss.c |  24 +++++++++++++-----------
 1 files changed, 13 insertions(+), 11 deletions(-)

diffs (65 lines):

diff -r fb0d5895054a -r 267012406009 sys/dev/fss.c
--- a/sys/dev/fss.c     Sun Jul 31 07:06:24 2016 +0000
+++ b/sys/dev/fss.c     Sun Jul 31 12:17:36 2016 +0000
@@ -1,4 +1,4 @@
-/*     $NetBSD: fss.c,v 1.94 2016/07/30 22:36:14 pgoyette Exp $        */
+/*     $NetBSD: fss.c,v 1.95 2016/07/31 12:17:36 hannken Exp $ */
 
 /*-
  * Copyright (c) 2003 The NetBSD Foundation, Inc.
@@ -36,7 +36,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: fss.c,v 1.94 2016/07/30 22:36:14 pgoyette Exp $");
+__KERNEL_RCSID(0, "$NetBSD: fss.c,v 1.95 2016/07/31 12:17:36 hannken Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -445,17 +445,20 @@
 
 /*
  * An error occurred reading or writing the snapshot or backing store.
  * If it is the first error log to console.
  * The caller holds the mutex.
  */
 static inline void
 fss_error(struct fss_softc *sc, const char *msg)
 {
 
-       if ((sc->sc_flags & (FSS_ACTIVE|FSS_ERROR)) == FSS_ACTIVE)
-               aprint_error_dev(sc->sc_dev, "snapshot invalid: %s\n", msg);
-       if ((sc->sc_flags & FSS_ACTIVE) == FSS_ACTIVE)
-               sc->sc_flags |= FSS_ERROR;
+       if ((sc->sc_flags & (FSS_ACTIVE | FSS_ERROR)) != FSS_ACTIVE)
+               return;
+
+       aprint_error_dev(sc->sc_dev, "snapshot invalid: %s\n", msg);
+       if ((sc->sc_flags & FSS_PERSISTENT) == 0)
+               fscow_disestablish(sc->sc_mount, fss_copy_on_write, sc);
+       sc->sc_flags |= FSS_ERROR;
 }
 
 /*
@@ -575,9 +578,8 @@
                if ((sc = device_lookup_private(&fss_cd, i)) == NULL)
                        continue;
                mutex_enter(&sc->sc_slock);
-               if ((sc->sc_flags & FSS_ACTIVE) != 0 &&
-                   sc->sc_mount == mp)
-                       fss_error(sc, "forced unmount");
+               if ((sc->sc_flags & FSS_ACTIVE) != 0 && sc->sc_mount == mp)
+                       fss_error(sc, "forced by unmount");
                mutex_exit(&sc->sc_slock);
        }
        mutex_exit(&fss_device_lock);
@@ -886,7 +888,7 @@
 fss_delete_snapshot(struct fss_softc *sc, struct lwp *l)
 {
 
-       if ((sc->sc_flags & FSS_PERSISTENT) == 0)
+       if ((sc->sc_flags & (FSS_PERSISTENT | FSS_ERROR)) == 0)
                fscow_disestablish(sc->sc_mount, fss_copy_on_write, sc);
 
        mutex_enter(&sc->sc_slock);
@@ -445,17 +445,20 @@
 /*
  * An error occurred reading or writing the snapshot or backing store.
- * If it is the first error log to console.
+ * If it is the first error log to console and disestablish cow handler.
  * The caller holds the mutex.
  */



Home | Main Index | Thread Index | Old Index